@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2026 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,9 @@
 #include "scriptingservice.h"
 #include "services/settingsservice.h"
 
+// Forward declaration to avoid circular dependency
+class QOwnNotesMarkdownTextEdit;
+
 using namespace std;
 
 QT_USE_NAMESPACE
@@ -34,11 +37,51 @@ OpenAiService::OpenAiService(QObject* parent) : QObject(parent) {
     initializeBackends();
     initializeCompleter(parent);
 
-    QObject::connect(_completer, &OpenAiCompleter::completed, this,
-                     [this](const QString& result) { qDebug() << "'result': " << result; });
-    QObject::connect(
-        _completer, &OpenAiCompleter::errorOccurred, this,
-        [this](const QString& errorString) { qDebug() << "'errorString': " << errorString; });
+    // Set up the global autocomplete callback that routes to the active editor
+    // This is done once here instead of in each widget to avoid widgets overwriting each other
+    setAutocompleteCallback([](const QString& result) {
+        qDebug() << "*** GLOBAL AUTOCOMPLETE CALLBACK RECEIVED, result length:" << result.length();
+        // Forward declaration to avoid circular dependency
+        class QOwnNotesMarkdownTextEdit;
+
+        // Get the active editor from the application property
+        // We can't use QMetaObject::invokeMethod with lambdas in Qt5
+        QObject* activeEditorObj = qApp->property("activeAutocompleteEditor").value<QObject*>();
+
+        if (activeEditorObj) {
+            qDebug() << "*** Calling onAiAutocompleteCompleted on active editor:"
+                     << activeEditorObj;
+            QMetaObject::invokeMethod(activeEditorObj, "onAiAutocompleteCompleted",
+                                      Qt::DirectConnection, Q_ARG(QString, result));
+        } else {
+            qDebug() << "*** No active editor found in callback!";
+        }
+    });
+
+    QObject::connect(_completer, &OpenAiCompleter::completed, this, [this](const QString& result) {
+        qDebug() << "'result': " << result;
+        qDebug() << "OpenAiService - processing autocomplete result";
+        qDebug() << "OpenAiService - this pointer:" << this;
+        qDebug() << "OpenAiService - Number of signal receivers:"
+                 << receivers(SIGNAL(autocompleteCompleted(QString)));
+        qDebug() << "OpenAiService - Has callback:" << (_autocompleteCallback != nullptr);
+
+        // Try callback first (direct call to active editor)
+        if (_autocompleteCallback) {
+            qDebug() << "OpenAiService - Calling autocomplete callback directly";
+            _autocompleteCallback(result);
+        } else {
+            qDebug() << "OpenAiService - No callback, emitting signal";
+            emit autocompleteCompleted(result);
+        }
+
+        qDebug() << "OpenAiService - autocomplete processing completed";
+    });
+    QObject::connect(_completer, &OpenAiCompleter::errorOccurred, this,
+                     [this](const QString& errorString) {
+                         qDebug() << "'errorString': " << errorString;
+                         emit autocompleteErrorOccurred(errorString);
+                     });
 }
 
 void OpenAiService::initializeCompleter(QObject* parent) {
@@ -80,9 +123,36 @@ void OpenAiService::deleteInstance() {
 
 void OpenAiService::initializeBackends() {
     _backendModels.clear();
-    _backendModels[QStringLiteral("openai")] =
-        QStringList{"gpt-4o",     "chatgpt-4o-latest", "gpt-4o-mini",   "o1-mini",
-                    "o1-preview", "gpt-4-turbo",       "gpt-3.5-turbo", "gpt-4"};
+    _backendModels[QStringLiteral("openai")] = QStringList{"gpt-5.1",
+                                                           "gpt-5",
+                                                           "gpt-5-mini",
+                                                           "gpt-5-nano",
+                                                           "gpt-5.1-chat-latest",
+                                                           "gpt-5-chat-latest",
+                                                           "gpt-5.1-codex-max",
+                                                           "gpt-5.1-codex",
+                                                           "gpt-5-codex",
+                                                           "gpt-5-pro",
+                                                           "gpt-4.1",
+                                                           "gpt-4.1-mini",
+                                                           "gpt-4.1-nano",
+                                                           "gpt-4o",
+                                                           "gpt-4o-2024-05-13",
+                                                           "gpt-4o-mini",
+                                                           "o1",
+                                                           "o1-pro",
+                                                           "o3-pro",
+                                                           "o3",
+                                                           "o3-deep-research",
+                                                           "o4-mini",
+                                                           "o4-mini-deep-research",
+                                                           "o3-mini",
+                                                           "o1-mini",
+                                                           "gpt-5.1-codex-mini",
+                                                           "codex-mini-latest",
+                                                           "gpt-5-search-api",
+                                                           "gpt-4o-mini-search-preview",
+                                                           "gpt-4o-search-preview"};
     _backendModels[QStringLiteral("groq")] =
         QStringList{"llama-3.1-8b-instant",
                     "deepseek-r1-distill-llama-70b",
@@ -93,9 +163,6 @@ void OpenAiService::initializeBackends() {
                     "groq/compound-mini",
                     "meta-llama/llama-4-maverick-17b-128e-instruct",
                     "meta-llama/llama-4-scout-17b-16e-instruct",
-                    "meta-llama/llama-guard-4-12b",
-                    "meta-llama/llama-prompt-guard-2-86m",
-                    "meta-llama/llama-prompt-guard-2-22m",
                     "moonshotai/kimi-k2-instruct",
                     "moonshotai/kimi-k2-instruct-0905",
                     "openai/gpt-oss-120b",
@@ -237,8 +304,10 @@ QString OpenAiService::getModelId() {
     // If not set yet try to read the settings
     if (this->_modelId.isEmpty()) {
         SettingsService settings;
-        this->_modelId =
-            settings.value(getCurrentModelSettingsKey(), _backendModels[getBackendId()]).toString();
+        // Get the first model as default if no model is set
+        const QStringList& models = getModelsForCurrentBackend();
+        QString defaultModel = models.isEmpty() ? QLatin1String("") : models.first();
+        this->_modelId = settings.value(getCurrentModelSettingsKey(), defaultModel).toString();
     }
 
     // If still not set get the first of the models
@@ -271,6 +340,31 @@ bool OpenAiService::getEnabled() {
     return settings.value(QStringLiteral("ai/enabled")).toBool();
 }
 
+bool OpenAiService::setAutocompleteEnabled(bool enabled) {
+    SettingsService settings;
+    settings.setValue(QStringLiteral("ai/autocompleteEnabled"), enabled);
+
+    return true;
+}
+
+bool OpenAiService::getAutocompleteEnabled() {
+    SettingsService settings;
+    return settings.value(QStringLiteral("ai/autocompleteEnabled"), false).toBool();
+}
+
+void OpenAiService::completeAsync(const QString& prompt) {
+    qDebug() << __func__ << " - called with prompt:" << prompt.left(50);
+
+    if (!getEnabled() || !getAutocompleteEnabled()) {
+        qDebug() << __func__ << " - not enabled, returning";
+        return;
+    }
+
+    qDebug() << __func__ << " - calling _completer->complete()";
+    _completer->complete(prompt);
+    qDebug() << __func__ << " - _completer->complete() called";
+}
+
 QString OpenAiService::complete(const QString& prompt) {
     if (!getEnabled()) {
         return tr("AI system is disabled. Please enable it in the main menu or the AI toolbar.");
@@ -279,6 +373,13 @@ QString OpenAiService::complete(const QString& prompt) {
     return _completer->completeSync(prompt);
 }
 
+void OpenAiService::setAutocompleteCallback(AutocompleteCallback callback) {
+    qDebug() << __func__
+             << " - Setting autocomplete callback, was:" << (_autocompleteCallback != nullptr)
+             << "now:" << (callback != nullptr);
+    _autocompleteCallback = callback;
+}
+
 OpenAiCompleter::OpenAiCompleter(QString apiKey, QString modelId, QString apiBaseUrl,
                                  QObject* parent)
     : QObject(parent),
