@@ -7,26 +7,38 @@
 #include <QApplication>
 #include <QClipboard>
 #include <QDebug>
+#include <QEvent>
 #include <QFont>
 #include <QFontDatabase>
+#include <QHelpEvent>
 #include <QJSEngine>
 #include <QKeyEvent>
 #include <QMenu>
 #include <QMimeData>
 #include <QRegularExpression>
+#include <QTextBlock>
 #include <QTextCursor>
 #include <QTextDocumentFragment>
+#include <QTextEdit>
+#include <QTextFormat>
 #include <QTimer>
+#include <QToolTip>
 
 #include "entities/notefolder.h"
 #include "helpers/qownspellchecker.h"
 #include "libraries/qmarkdowntextedit/linenumberarea.h"
 #include "mainwindow.h"
+#include "services/markdownlspclient.h"
 #include "services/nextclouddeckservice.h"
 #include "services/openaiservice.h"
 #include "services/scriptingservice.h"
 #include "services/settingsservice.h"
 #include "utils/urlhandler.h"
+#include "version.h"
+
+namespace {
+constexpr int kMarkdownLspDiagnosticProperty = QTextFormat::UserProperty + 1;
+}
 
 // Initialize static member
 QOwnNotesMarkdownTextEdit *QOwnNotesMarkdownTextEdit::_activeAutocompleteEditor = nullptr;
@@ -50,6 +62,12 @@ QOwnNotesMarkdownTextEdit::QOwnNotesMarkdownTextEdit(QWidget *parent)
     connect(_aiAutocompleteTimer, &QTimer::timeout, this,
             &QOwnNotesMarkdownTextEdit::requestAiAutocomplete);
 
+    _markdownLspChangeTimer = new QTimer(this);
+    _markdownLspChangeTimer->setSingleShot(true);
+    _markdownLspChangeTimer->setInterval(200);
+    connect(_markdownLspChangeTimer, &QTimer::timeout, this,
+            &QOwnNotesMarkdownTextEdit::sendMarkdownLspChange);
+
     // NOTE: Callback registration is now done globally in OpenAiService constructor
     // We just need to register this editor as active if it's a note editor
     if (!parent || parent->objectName() != QStringLiteral("LogWidget")) {
@@ -97,6 +115,10 @@ QOwnNotesMarkdownTextEdit::QOwnNotesMarkdownTextEdit(QWidget *parent)
             clearAiAutocompleteSuggestion();
             _aiAutocompleteTimer->start();
         }
+
+        if (_markdownLspEnabled) {
+            scheduleMarkdownLspChange();
+        }
     });
 
     SettingsService settings;
@@ -140,6 +162,8 @@ QOwnNotesMarkdownTextEdit::QOwnNotesMarkdownTextEdit(QWidget *parent)
     setContextMenuPolicy(Qt::CustomContextMenu);
     connect(this, &QOwnNotesMarkdownTextEdit::customContextMenuRequested, this,
             &QOwnNotesMarkdownTextEdit::onContextMenu);
+
+    applyMarkdownLspSettings();
 }
 
 /*
@@ -603,6 +627,17 @@ void QOwnNotesMarkdownTextEdit::onAutoCompleteRequested() {
         return;
     }
 
+    if (_markdownLspEnabled && _markdownLspClient && !_markdownLspUri.isEmpty()) {
+        const QTextCursor cursor = textCursor();
+        const int line = cursor.blockNumber();
+        const int character = cursor.positionInBlock();
+        _markdownLspCompletionRequestId =
+            _markdownLspClient->requestCompletion(_markdownLspUri, line, character);
+        if (_markdownLspCompletionRequestId >= 0) {
+            return;
+        }
+    }
+
     // try to open a link at the cursor position
     if (openLinkAtCursorPosition()) {
         MainWindow::instance()->showStatusBarMessage(
@@ -952,6 +987,8 @@ void QOwnNotesMarkdownTextEdit::updateSettings() {
 
     _centerCursor = settings.value(QStringLiteral("Editor/centerCursor")).toBool();
     QMarkdownTextEdit::updateSettings();
+
+    applyMarkdownLspSettings();
 }
 
 void QOwnNotesMarkdownTextEdit::onContextMenu(QPoint pos) {
@@ -1614,6 +1651,262 @@ QOwnNotesMarkdownTextEdit *QOwnNotesMarkdownTextEdit::getActiveEditorForAutocomp
 
 QOwnNotesMarkdownTextEdit::~QOwnNotesMarkdownTextEdit() {
     qDebug() << "*** QOwnNotesMarkdownTextEdit DESTROYED ***" << this << objectName();
+    closeMarkdownLspDocument();
     // Unregister if this was the active editor
     unregisterAsActiveEditor();
 }
+
+void QOwnNotesMarkdownTextEdit::setMarkdownLspDocumentPath(const QString &filePath,
+                                                           const QString &text) {
+    if (!_markdownLspEnabled) {
+        return;
+    }
+
+    const QString uri = QUrl::fromLocalFile(filePath).toString();
+    if (uri.isEmpty()) {
+        return;
+    }
+
+    if (_markdownLspUri == uri) {
+        return;
+    }
+
+    if (_markdownLspClient && !_markdownLspUri.isEmpty()) {
+        _markdownLspClient->didClose(_markdownLspUri);
+    }
+
+    _markdownLspUri = uri;
+    _markdownLspVersion = 1;
+
+    if (_markdownLspClient) {
+        _markdownLspClient->didOpen(_markdownLspUri, QStringLiteral("markdown"), text,
+                                    _markdownLspVersion);
+    }
+}
+
+void QOwnNotesMarkdownTextEdit::closeMarkdownLspDocument() {
+    if (_markdownLspClient && !_markdownLspUri.isEmpty()) {
+        _markdownLspClient->didClose(_markdownLspUri);
+    }
+
+    _markdownLspUri.clear();
+    _markdownLspVersion = 0;
+    _markdownLspDiagnosticsSelections.clear();
+    applyMarkdownLspDiagnosticsSelections();
+}
+
+void QOwnNotesMarkdownTextEdit::applyMarkdownLspSettings() {
+    SettingsService settings;
+    const bool enabled =
+        settings.value(QStringLiteral("Editor/markdownLspEnabled"), false).toBool();
+    const QString command =
+        settings.value(QStringLiteral("Editor/markdownLspCommand"), QStringLiteral("marksman"))
+            .toString();
+    const QStringList arguments =
+        settings.value(QStringLiteral("Editor/markdownLspArguments")).toStringList();
+
+    if (!enabled) {
+        _markdownLspEnabled = false;
+        if (_markdownLspClient) {
+            closeMarkdownLspDocument();
+            _markdownLspClient->shutdown();
+        }
+        return;
+    }
+
+    if (!_markdownLspClient) {
+        _markdownLspClient = new MarkdownLspClient(this);
+        connect(_markdownLspClient, &MarkdownLspClient::completionReceived, this,
+                &QOwnNotesMarkdownTextEdit::showMarkdownLspCompletions);
+        connect(_markdownLspClient, &MarkdownLspClient::diagnosticsReceived, this,
+                &QOwnNotesMarkdownTextEdit::showMarkdownLspDiagnostics);
+        connect(_markdownLspClient, &MarkdownLspClient::errorMessage, this,
+                [this](const QString &message) {
+                    if (!message.trimmed().isEmpty()) {
+                        qWarning() << "Markdown LSP:" << message.trimmed();
+                    }
+                });
+        connect(_markdownLspClient, &MarkdownLspClient::serverInitialized, this, []() {
+            if (auto *mw = MainWindow::instance()) {
+                mw->showStatusBarMessage(tr("Markdown LSP server connected"), 3000);
+            }
+        });
+    }
+
+    _markdownLspClient->setServerCommand(command, arguments);
+    if (_markdownLspClient->start()) {
+        const QString rootPath = NoteFolder::currentLocalPath();
+        _markdownLspClient->initialize(rootPath, QStringLiteral("QOwnNotes"),
+                                       QStringLiteral(VERSION));
+    }
+
+    _markdownLspEnabled = true;
+
+    if (!_markdownLspUri.isEmpty()) {
+        _markdownLspVersion = 1;
+        _markdownLspClient->didOpen(_markdownLspUri, QStringLiteral("markdown"), toPlainText(),
+                                    _markdownLspVersion);
+    }
+}
+
+void QOwnNotesMarkdownTextEdit::scheduleMarkdownLspChange() {
+    if (!_markdownLspEnabled || !_markdownLspClient || _markdownLspUri.isEmpty()) {
+        return;
+    }
+
+    _markdownLspPendingText = toPlainText();
+    _markdownLspChangeTimer->start();
+}
+
+void QOwnNotesMarkdownTextEdit::sendMarkdownLspChange() {
+    if (!_markdownLspEnabled || !_markdownLspClient || _markdownLspUri.isEmpty()) {
+        return;
+    }
+
+    _markdownLspVersion++;
+    _markdownLspClient->didChange(_markdownLspUri, _markdownLspPendingText, _markdownLspVersion);
+}
+
+void QOwnNotesMarkdownTextEdit::showMarkdownLspCompletions(int requestId,
+                                                           const QStringList &items) {
+    if (requestId != _markdownLspCompletionRequestId || _markdownLspCompletionRequestId == -1) {
+        return;
+    }
+
+    QMenu menu;
+    for (const QString &text : items) {
+        auto *action = menu.addAction(text);
+        action->setData(text);
+        action->setWhatsThis(QStringLiteral("autocomplete"));
+    }
+
+    _markdownLspCompletionRequestId = -1;
+
+    if (menu.actions().isEmpty()) {
+        return;
+    }
+
+    QPoint globalPos = mapToGlobal(cursorRect().bottomRight());
+    globalPos.setY(globalPos.y() + viewportMargins().top());
+    globalPos.setX(globalPos.x() + viewportMargins().left());
+
+    QAction *selectedItem = menu.exec(globalPos);
+    if (!selectedItem) {
+        return;
+    }
+
+    const QString text = selectedItem->data().toString();
+    if (text.isEmpty()) {
+        return;
+    }
+
+    QTextCursor c = textCursor();
+    c.movePosition(QTextCursor::StartOfWord, QTextCursor::KeepAnchor);
+    c.insertText(text + QStringLiteral(" "));
+}
+
+void QOwnNotesMarkdownTextEdit::showMarkdownLspDiagnostics(
+    const QString &uri, const QVector<MarkdownLspClient::Diagnostic> &diagnostics) {
+    if (uri != _markdownLspUri) {
+        return;
+    }
+
+    _markdownLspDiagnosticsSelections.clear();
+    for (const MarkdownLspClient::Diagnostic &diagnostic : diagnostics) {
+        if (diagnostic.message.isEmpty()) {
+            continue;
+        }
+
+        const QTextBlock startBlock = document()->findBlockByNumber(diagnostic.range.startLine);
+        if (!startBlock.isValid()) {
+            continue;
+        }
+
+        const QTextBlock endBlock = document()->findBlockByNumber(diagnostic.range.endLine);
+        if (!endBlock.isValid()) {
+            continue;
+        }
+
+        const int startPosition = startBlock.position() + diagnostic.range.startCharacter;
+        const int endPosition = endBlock.position() + diagnostic.range.endCharacter;
+        if (endPosition <= startPosition) {
+            continue;
+        }
+
+        // Choose underline color based on LSP severity:
+        // 1 = Error, 2 = Warning, 3 = Information, 4 = Hint
+        QColor underlineColor;
+        switch (diagnostic.severity) {
+            case 1:
+                underlineColor = QColor(214, 68, 68);    // Red for errors
+                break;
+            case 2:
+                underlineColor = QColor(210, 140, 30);    // Orange for warnings
+                break;
+            case 3:
+                underlineColor = QColor(80, 140, 210);    // Blue for information
+                break;
+            case 4:
+                underlineColor = QColor(100, 170, 100);    // Green for hints
+                break;
+            default:
+                underlineColor = QColor(214, 68, 68);    // Default to red
+                break;
+        }
+
+        QTextCursor cursor(document());
+        cursor.setPosition(startPosition);
+        cursor.setPosition(endPosition, QTextCursor::KeepAnchor);
+
+        QTextEdit::ExtraSelection selection;
+        selection.cursor = cursor;
+        selection.format.setUnderlineStyle(QTextCharFormat::WaveUnderline);
+        selection.format.setUnderlineColor(underlineColor);
+        selection.format.setToolTip(diagnostic.message);
+        selection.format.setProperty(kMarkdownLspDiagnosticProperty, true);
+        _markdownLspDiagnosticsSelections.append(selection);
+    }
+
+    applyMarkdownLspDiagnosticsSelections();
+}
+
+bool QOwnNotesMarkdownTextEdit::viewportEvent(QEvent *event) {
+    if (event->type() == QEvent::ToolTip && !_markdownLspDiagnosticsSelections.isEmpty()) {
+        auto *helpEvent = static_cast<QHelpEvent *>(event);
+        const QPoint pos = helpEvent->pos();
+        const int cursorPos = cursorForPosition(pos).position();
+
+        // Find the first diagnostic selection that covers the cursor position
+        for (const QTextEdit::ExtraSelection &selection :
+             qAsConst(_markdownLspDiagnosticsSelections)) {
+            const int selStart = selection.cursor.selectionStart();
+            const int selEnd = selection.cursor.selectionEnd();
+            if (cursorPos >= selStart && cursorPos <= selEnd) {
+                const QString tip = selection.format.toolTip();
+                if (!tip.isEmpty()) {
+                    QToolTip::showText(helpEvent->globalPos(), tip, this);
+                    event->accept();
+                    return true;
+                }
+            }
+        }
+
+        QToolTip::hideText();
+    }
+    return QMarkdownTextEdit::viewportEvent(event);
+}
+
+void QOwnNotesMarkdownTextEdit::applyMarkdownLspDiagnosticsSelections() {
+    const QList<QTextEdit::ExtraSelection> selections = extraSelections();
+    QList<QTextEdit::ExtraSelection> filteredSelections;
+    filteredSelections.reserve(selections.size());
+    for (const QTextEdit::ExtraSelection &selection : selections) {
+        if (selection.format.property(kMarkdownLspDiagnosticProperty).toBool()) {
+            continue;
+        }
+        filteredSelections.append(selection);
+    }
+
+    filteredSelections += _markdownLspDiagnosticsSelections;
+    setExtraSelections(filteredSelections);
+}
